package jbpm;

import org.jboss.seam.Component;
import org.jboss.seam.log.Log;
import org.jboss.seam.log.Logging;
import org.jbpm.graph.def.ActionHandler;
import org.jbpm.graph.exe.ExecutionContext;

import application.model.NetworkSwitch;

import cisco.ParseSwitchInterface;

public class CiscoConnect implements ActionHandler {

	/**
	 * Logger for this class
	 */
	private static final Log log = Logging.getLog(CiscoConnect.class);

	private static final long serialVersionUID = 1L;

	public void execute(ExecutionContext executionContext) throws Exception {

		log.debug("Launching CiscoConnect");

		ParseSwitchInterface cisco = (ParseSwitchInterface) Component
				.getInstance("cisco");
		executionContext.getJbpmContext().getSession().beginTransaction();
		try {
			log.debug("Variables: " + executionContext.getContextInstance().getVariables());
			log.debug("Key: " + executionContext.getProcessInstance().getKey());
			NetworkSwitch networkswitch = (NetworkSwitch)executionContext.getVariable("networkswitch");			
			executionContext.setVariable("networkswitch", networkswitch);
			cisco.connect(networkswitch);
			executionContext.setVariable("networkswitch", networkswitch);
			executionContext.leaveNode("parseInterfaces");
		} catch (Exception e) {		
			executionContext.setVariable("error", e.getMessage());
			log.error("Exception occurred in CiscoConnect: " +  e.getMessage());
			executionContext.getNode().raiseException(e, executionContext);			
		}

		log.debug("Ending CiscoConnect");

	}
}
