package cisco;

import java.io.Serializable;
import java.text.SimpleDateFormat;
import java.util.ArrayList;
import java.util.Calendar;
import java.util.Date;
import java.util.HashMap;
import java.util.Iterator;
import java.util.List;
import java.util.Locale;
import java.util.Map;

import javax.ejb.Stateless;
import javax.ejb.TransactionAttribute;
import javax.ejb.TransactionAttributeType;
import javax.persistence.EntityManager;
import javax.persistence.NoResultException;
import javax.persistence.NonUniqueResultException;

import org.jboss.seam.ScopeType;
import org.jboss.seam.annotations.In;
import org.jboss.seam.annotations.Logger;
import org.jboss.seam.annotations.Name;
import org.jboss.seam.annotations.Out;
import org.jboss.seam.annotations.Scope;
import org.jboss.seam.annotations.Transactional;
import org.jboss.seam.bpm.BusinessProcess;
import org.jboss.seam.log.Log;

import telnet.TelnetAction;

import application.model.NetworkSwitch;

import crypto.CryptoLibrary;

@Stateless
@Scope(ScopeType.CONVERSATION)
@Name("cisco")
public class ParseSwitch implements Serializable, ParseSwitchInterface {

	private static final long serialVersionUID = -886357390149851922L;

	@Logger
	Log log;

	@Out(required = false, scope = ScopeType.BUSINESS_PROCESS)
	@In(required = false, scope = ScopeType.BUSINESS_PROCESS)
	NetworkSwitch networkswitch;

	@Out(required = false, scope = ScopeType.BUSINESS_PROCESS)
	Date scanTime;

	@In
	EntityManager entityManager;

	@In(required = false)
	BusinessProcess businessProcess;

	public void endScan(NetworkSwitch networkswitch) {
		log.debug("Executing endScan...");
		log.debug("Ended scan for switch " + networkswitch.getName() + " ("
				+ networkswitch.getId() + ")");
	}

	public void connect(NetworkSwitch networkswitch) throws Exception {
		log.debug("Executing connect...");
		scanTime = new Date();
		log.debug("Connecting to switch " + networkswitch.getName() + " ("
				+ networkswitch.getId() + ")");

		try {
			//Connect to the networkswitch via telnet and gather the necessary information
			networkswitch.setMacAddress("00:11:22:33:44:55");			
			networkswitch.setSerial("serial number read via telnet");
			networkswitch.setName("switch name read via telnet");
			networkswitch.setLastScan(scanTime);
			networkswitch.setInterfaceInfo("interface info read via telnet");
			networkswitch.setVlanInfo("vlan info read via telnet");
			networkswitch.setMacInfo("mac info read via telnet");

			entityManager.merge(networkswitch);

		} catch (Exception e) {

			throw e;
		}
		log.debug("Collected connect information for switch "
				+ networkswitch.getName() + " (" + networkswitch.getId() + ")");

	}

	public void parseInterfaces(NetworkSwitch networkswitch) throws Exception {
		log.debug("Started parseInterfaces for switch "
				+ networkswitch.getName() + " (" + networkswitch.getId() + ")");

		try {
			// Here we are going to process the information which has been read in the connect node
			
			
			}
		} catch (Exception e) {
			throw e;
		}
		log.debug("Completed parseInterfaces for switch "
				+ networkswitch.getName() + " (" + networkswitch.getId() + ")");
	}

	public void parseVlans(NetworkSwitch networkswitch) throws Exception {
		log.debug("Started parseVlans for switch " + networkswitch.getName()
				+ " (" + networkswitch.getId() + ")");
		try {
			// Here we are going to process the vlan information which has been read in the connect node

			}
		} catch (Exception e) {

			throw e;
		}
		log.debug("Completed parseVlans for switch " + networkswitch.getName()
				+ " (" + networkswitch.getId() + ")");
	}

	public void parseMacs(NetworkSwitch networkswitch) throws Exception {
		log.debug("Started parseMacs for switch " + networkswitch.getName()
				+ " (" + networkswitch.getId() + ")");

		try {
			// Here we are going to process the mac information which has been read in the connect node

			}
		} catch (Exception e) {
			throw e;
		}
		log.debug("Completed parseMacs for switch " + networkswitch.getName()
				+ " (" + networkswitch.getId() + ")");
	}

	@Transactional
	@TransactionAttribute	(value=TransactionAttributeType.NOT_SUPPORTED)
	public void linkMacPort(NetworkSwitch networkswitch) throws Exception {
		log.debug("Started linkMacPort for switch " + networkswitch.getName()
				+ " (" + networkswitch.getId() + ")");
		try {
			// We are going to link the mac address to the
			// appropriate port

		} catch (Exception e) {
			throw e;
		}
		endScan(networkswitch);
		log.debug("Completed linkMacPort for switch " + networkswitch.getName()
				+ " (" + networkswitch.getId() + ")");
	}

}
