package application.action;

import java.io.Serializable;
import java.util.Iterator;
import java.util.List;

import javax.ejb.TransactionAttribute;
import javax.ejb.TransactionAttributeType;
import javax.persistence.EntityManager;

import org.jboss.seam.ScopeType;
import org.jboss.seam.annotations.End;
import org.jboss.seam.annotations.In;
import org.jboss.seam.annotations.Logger;
import org.jboss.seam.annotations.Name;
import org.jboss.seam.annotations.Out;
import org.jboss.seam.annotations.Scope;
import org.jboss.seam.annotations.Transactional;
import org.jboss.seam.international.StatusMessage;
import org.jboss.seam.international.StatusMessages;
import org.jboss.seam.log.Log;
import org.jbpm.JbpmContext;
import org.jbpm.db.GraphSession;
import org.jbpm.graph.def.ProcessDefinition;
import org.jbpm.graph.exe.ProcessInstance;
import org.jbpm.graph.exe.Token;

import application.model.NetworkSwitch;

@Name("scanAction")
@Scope(ScopeType.CONVERSATION)
public class ScanAction implements Serializable, ScanActionInterface {

	/**
	 * 
	 */
	private static final long serialVersionUID = 1L;

	@In
	private EntityManager entityManager;

	@In(required = false)
	JbpmContext jbpmContext;	

	@In(required = false)
	private StatusMessages statusMessages;

	@SuppressWarnings("unused")
	@Out(required = false, scope=ScopeType.BUSINESS_PROCESS)
	private NetworkSwitch networkswitch;
	
	@Logger
	Log log;

	@SuppressWarnings("unchecked")
	@Transactional
	@End
	public void scanAll() {

		log.info("Launching scan for ALL network switches");
		
		List<NetworkSwitch> networkswitches = (List<NetworkSwitch>) entityManager.createQuery("select n from NetworkSwitch n").getResultList();

		GraphSession graphSession = jbpmContext.getGraphSession();
		ProcessDefinition processDefinition = graphSession.findLatestProcessDefinition("scanswitch");

		Iterator<NetworkSwitch> i = networkswitches.iterator();
		int count=0;
		while (i.hasNext()) {
			count++;

			log.info("Processing switch "+ count + " of " +networkswitches.size() );
			
			NetworkSwitch networkswitch = i.next();
			try {
			ProcessInstance processInstance = jbpmContext.getProcessInstance(
					processDefinition, networkswitch.getId().toString());
			if (processInstance == null) {
				processInstance = new ProcessInstance(processDefinition);
				processInstance.setKey(networkswitch.getId().toString());
			}
			processInstance.getContextInstance().setVariable("networkswitch",
					networkswitch);
			Token token = processInstance.getRootToken();
			token.signal();
			jbpmContext.save(processInstance);
			} catch (Exception e) {
				log.error("Something bad happened! " + e.getMessage());
			}
		}

	}

	@Transactional
	@End
	public void scan(NetworkSwitch networkswitch) {
		try {
			this.networkswitch = networkswitch;
			log.info("Launching scan for networkswitch "
					+ networkswitch.getName() + " (" + networkswitch.getId()
					+ ")");
			GraphSession graphSession = jbpmContext.getGraphSession();
			ProcessDefinition processDefinition = graphSession
					.findLatestProcessDefinition("scanswitch");
			ProcessInstance processInstance = jbpmContext.getProcessInstance(
					processDefinition, networkswitch.getId().toString());
			if (processInstance == null) {
				processInstance = new ProcessInstance(processDefinition);
				processInstance.setKey(networkswitch.getId().toString());
			}
			processInstance.getContextInstance().setVariable("networkswitch",
					networkswitch);

			jbpmContext.save(processInstance);
			processInstance.getRootToken().signal();
			statusMessages.add(StatusMessage.Severity.INFO, "Launched network scan for " + networkswitch.getName() +  " ("+networkswitch.getId()+")");
		} catch (Exception e) {
			log.error("Something bad happened! " + e.getMessage());
		}
	}

}
